@@ -23,6 +23,9 @@
 #' centered on row, "r" for right justification, or "l" for left justification. Default
 #' Value is "l"  If using html, the alignment can be set by using the label_row_css
 #' parameter.
+#' @param latex_wrap_text T/F for wrapping long text. Default is off. Whenever
+#' it is turned on, the table will take up the entire line. It's recommended
+#' to use this with full_width in kable_styling.
 #' @param colnum A numeric that determines how many columns the text should span.
 #' The default setting will have the text span the entire length.
 #' @param bold A T/F value to control whether the text should be bolded.
@@ -32,11 +35,24 @@
 #' @param hline_after A replicate of `hline.after` in xtable. It
 #' addes a hline after the row
 #' @param extra_latex_after Extra LaTeX text to be added after the row.
-#' @param indent A T?F value to control whether list items are indented.
+#' @param indent A T/F value to control whether list items are indented.
+#' @param monospace T/F value to control whether the text of the
+#' selected column need to be monospaced (verbatim)
+#' @param underline T/F value to control whether the text of the
+#' selected row need to be underlined
+#' @param strikeout T/F value to control whether the text of the
+#' selected row need to be striked out.
+#' @param color A character string for column text color. Here please
+#' pay attention to the differences in color codes between HTML and LaTeX.
+#' @param background A character string for column background color. Here please
+#' pay attention to the differences in color codes between HTML and LaTeX.
 #'
-#' @examples x <- knitr::kable(head(mtcars), "html")
+#' @examples
+#' \dontrun{
+#' x <- knitr::kable(head(mtcars), "html")
 #' # Put Row 2 to Row 5 into a Group and label it as "Group A"
 #' pack_rows(x, "Group A", 2, 5)
+#' }
 #'
 #' @export
 group_rows <- function(kable_input, group_label = NULL,
@@ -44,13 +60,17 @@ group_rows <- function(kable_input, group_label = NULL,
                        index = NULL,
                        label_row_css = "border-bottom: 1px solid;",
                        latex_gap_space = "0.3em",
-                       escape = TRUE, latex_align = "l", colnum = NULL,
+                       escape = TRUE, latex_align = "l",
+                       latex_wrap_text = FALSE,
+                       colnum = NULL,
                        bold = TRUE,
                        italic = FALSE,
                        hline_before = FALSE,
                        hline_after = FALSE,
                        extra_latex_after = NULL,
-                       indent = TRUE) {
+                       indent = TRUE,
+                       monospace = FALSE, underline = FALSE, strikeout = FALSE,
+                       color = NULL, background = NULL) {
 
   kable_format <- attr(kable_input, "format")
   if (!kable_format %in% c("html", "latex")) {
@@ -59,29 +79,35 @@ group_rows <- function(kable_input, group_label = NULL,
             "for details.")
     return(kable_input)
   }
+
   if (is.null(index)) {
     if (kable_format == "html") {
-      if(!missing(latex_align)) warning("latex_align parameter is not used in HTML Mode,
+      if (!missing(latex_align)) warning("latex_align parameter is not used in HTML Mode,
                                     use label_row_css instead.")
       return(group_rows_html(kable_input, group_label, start_row, end_row,
-                             label_row_css, escape, colnum, indent))
-    }
+                             label_row_css, escape, colnum, indent,
+                             bold, italic, monospace, underline, strikeout,
+                             color, background))}
     if (kable_format == "latex") {
       return(group_rows_latex(kable_input, group_label, start_row, end_row,
                               latex_gap_space, escape, latex_align, colnum,
                               bold, italic, hline_before, hline_after,
-                              extra_latex_after, indent))
+                              extra_latex_after, indent, latex_wrap_text,
+                              monospace, underline, strikeout,
+                              color, background))
     }
   } else {
     index <- group_row_index_translator(index)
     out <- kable_input
     if (kable_format == "html") {
       for (i in 1:nrow(index)) {
-        if(!missing(latex_align)) warning("latex_align parameter is not used in HTML Mode,
+        if (!missing(latex_align)) warning("latex_align parameter is not used in HTML Mode,
                                     use label_row_css instead.")
         out <- group_rows_html(out, index$header[i],
                                index$start[i], index$end[i],
-                               label_row_css, escape, colnum, indent)
+                               label_row_css, escape, colnum, indent,
+                               bold, italic, monospace, underline, strikeout,
+                               color, background)
       }
     }
     if (kable_format == "latex") {
@@ -90,7 +116,9 @@ group_rows <- function(kable_input, group_label = NULL,
                                index$start[i], index$end[i],
                                latex_gap_space, escape, latex_align, colnum,
                                bold, italic, hline_before, hline_after,
-                               extra_latex_after, indent)
+                               extra_latex_after, indent, latex_wrap_text,
+                               monospace, underline, strikeout,
+                               color, background)
       }
     }
     return(out)
@@ -107,7 +135,9 @@ group_row_index_translator <- function(index) {
 }
 
 group_rows_html <- function(kable_input, group_label, start_row, end_row,
-                            label_row_css, escape, colnum, indent) {
+                            label_row_css, escape, colnum, indent,
+                            bold, italic, monospace, underline, strikeout,
+                            color, background) {
   kable_attrs <- attributes(kable_input)
   kable_xml <- read_kable_as_xml(kable_input)
   kable_tbody <- xml_tpart(kable_xml, "tbody")
@@ -121,6 +151,10 @@ group_rows_html <- function(kable_input, group_label, start_row, end_row,
   if (!is.null(group_header_rows)) {
     group_seq <- positions_corrector(group_seq, group_header_rows,
                                      length(xml_children(kable_tbody)))
+    # Update the old group_header_rows attribute with their new positions
+    kable_attrs$group_header_rows <- ifelse(kable_attrs$group_header_rows > group_seq[1],
+                                            kable_attrs$group_header_rows+1,
+                                            kable_attrs$group_header_rows)
   }
 
   # Insert a group header row
@@ -128,11 +162,50 @@ group_rows_html <- function(kable_input, group_label, start_row, end_row,
   kable_ncol <- ifelse(is.null(colnum),
                        length(xml_children(starting_node)),
                        colnum)
+
+  if (bold) group_label <- paste0("<strong>", group_label, "</strong>")
+  if (italic) group_label <- paste0("<em>", group_label, "</em>")
+
+  if (label_row_css == "border-bottom: 1px solid;") {
+    if (!is.null(attr(kable_input, "lightable_class"))) {
+      lightable_class <- attr(kable_input, "lightable_class")
+      if (lightable_class %in% c(
+        "lightable-classic", "lightable-classic-2", "lightable-minimal")) {
+        label_row_css <- "border-bottom: 0;"
+      }
+      if (lightable_class %in% c("lightable-paper")) {
+        label_row_css <- "border-bottom: 1px solid #00000020;"
+      }
+      if (lightable_class %in% c("lightable-material")) {
+        label_row_css <- "border-bottom: 1px solid #eee; "
+      }
+      if (lightable_class %in% c("lightable-material-dark")) {
+        label_row_css <- "border-bottom: 1px solid #FFFFFF12; color: #FFFFFF60;"
+      }
+    }
+  }
+  if (monospace) {
+    label_row_css <- paste0(label_row_css, "font-family: monospace;")
+  }
+  if (underline) {
+    label_row_css <- paste0(label_row_css, "text-decoration: underline;")
+  }
+  if (strikeout) {
+    label_row_css <- paste0(label_row_css, "text-decoration: line-through;")
+  }
+  if (!is.null(color)) {
+    label_row_css <- paste0(label_row_css, "color: ", html_color(color),
+                            " !important;")
+  }
+  if (!is.null(background)) {
+    label_row_css <- paste0(label_row_css, "background-color: ",
+                            html_color(background), " !important;")
+  }
+
   group_header_row_text <- paste0(
     '<tr groupLength="', length(group_seq), '"><td colspan="', kable_ncol,
-    '" style="', label_row_css, '"><strong>', group_label,
-    "</strong></td></tr>"
-  )
+    '" style="', label_row_css, '">', group_label, "</td></tr>")
+
   group_header_row <- read_xml(group_header_row_text, options = "COMPACT")
   xml_add_sibling(starting_node, group_header_row, .where = "before")
 
@@ -148,8 +221,10 @@ group_rows_html <- function(kable_input, group_label, start_row, end_row,
 
 group_rows_latex <- function(kable_input, group_label, start_row, end_row,
                              gap_space, escape, latex_align, colnum,
-                             bold = T, italic = F, hline_before = F ,hline_after = F,
-                             extra_latex_after = NULL, indent) {
+                             bold = T, italic = F, hline_before = F, hline_after = F,
+                             extra_latex_after = NULL, indent, latex_wrap_text = F,
+                             monospace = F, underline = F, strikeout = F,
+                             color = NULL, background = NULL) {
   table_info <- magic_mirror(kable_input)
   out <- solve_enc(kable_input)
 
@@ -161,40 +236,76 @@ group_rows_latex <- function(kable_input, group_label, start_row, end_row,
 
   if (escape) {
     group_label <- input_escape(group_label, latex_align)
+  } else {
+    group_label <- sim_double_escape(group_label)
   }
 
   if (bold) {
     group_label <- paste0("\\\\textbf{", group_label, "}")
   }
+
   if (italic) group_label <- paste0("\\\\textit{", group_label, "}")
+
+  if (monospace) {
+    group_label <- paste0("\\\\ttfamily\\{", group_label, "\\}")
+  }
+  if (underline) {
+    group_label <- paste0("\\\\underline\\{", group_label, "\\}")
+  }
+  if (strikeout) {
+    group_label <- paste0("\\\\sout\\{", group_label, "\\}")
+  }
+  if (!is.null(color)) {
+    group_label <- paste0("\\\\textcolor", latex_color(color), "\\{",
+                              group_label, "\\}")
+  }
+  if (!is.null(background)) {
+    group_label <- paste0("\\\\cellcolor", latex_color(background), "\\{",
+                              group_label, "\\}")
+  }
   # Add group label
-  if (table_info$booktabs) {
-    rowtext <- table_info$contents[start_row + table_info$position_offset]
-    pre_rowtext <- paste0(
-      "\\\\addlinespace[", gap_space, "]\n",
-      ifelse(hline_before,"\\\\hline\n", ""),
-      "\\\\multicolumn{", ifelse(is.null(colnum),
-                                 table_info$ncol,
-                                 colnum),
-      "}{", latex_align, "}{", group_label,
-      "}\\\\\\\\\n", ifelse(hline_after, "\\\\hline\n", '')
-    )
-  } else {
-    rowtext <- table_info$contents[start_row + 1]
-    rowtext <- paste0("\\\\hline\n", rowtext)
-    pre_rowtext <- paste0(
-      "\\\\hline\n\\\\multicolumn{", table_info$ncol, "}{", latex_align,"}{",
-      group_label, "}\\\\\\\\\n"
+  if (latex_wrap_text) {
+    latex_align <- switch(
+      latex_align,
+      "l" = "p{\\\\linewidth}",
+      "c" = ">{\\\\centering\\\\arraybackslash}p{\\\\linewidth}",
+      "r" = ">{\\\\centering\\\\arraybackslash}p{\\\\linewidth}"
     )
   }
+
+
+  rowtext <- table_info$contents[start_row + table_info$position_offset]
+  if (table_info$booktabs) {
+    pre_rowtext <- paste0("\\\\addlinespace[", gap_space, "]\n")
+  } else {
+    pre_rowtext <- ''
+    hline_after <- TRUE
+  }
+  pre_rowtext <- paste0(
+    pre_rowtext,
+    ifelse(hline_before,"\\\\hline\n", ""),
+    "\\\\multicolumn{", ifelse(is.null(colnum),
+                               table_info$ncol,
+                               colnum),
+    "}{", latex_align,"}{", group_label,
+    "}\\\\\\\\\n", ifelse(hline_after, "\\\\hline\n", '')
+  )
   if(!is.null(extra_latex_after)){
     pre_rowtext <- paste0(pre_rowtext,
                       regex_escape(extra_latex_after, double_backslash = TRUE))
   }
   new_rowtext <- paste0(pre_rowtext, rowtext)
-  out <- sub(paste0(rowtext, "\\\\\\\\\n"),
-  	         paste0(new_rowtext, "\\\\\\\\\n"),
-  	         out)
+  if (start_row + 1 == table_info$nrow &
+      !is.null(table_info$repeat_header_latex) & table_info$booktabs) {
+    out <- sub(paste0(rowtext, "\\\\\\\\\\*\n"),
+               paste0(new_rowtext, "\\\\\\\\\\*\n"),
+               out)
+  } else {
+    out <- sub(paste0(rowtext, "\\\\\\\\\n"),
+               paste0(new_rowtext, "\\\\\\\\\n"),
+               out)
+  }
+
   out <- gsub("\\\\addlinespace\n", "", out)
   out <- structure(out, format = "latex", class = "knitr_kable")
   table_info$group_rows_used <- TRUE
@@ -215,6 +326,7 @@ group_rows_latex <- function(kable_input, group_label, start_row, end_row,
 #'
 #' @export
 auto_index <- function(x) {
+  if (is.factor(x)) x <- as.character(x)
   x_rle <- rle(x)
   index <- x_rle$lengths
   names(index) <- x_rle$values
