@@ -34,8 +34,11 @@
 #' @param extra_latex_after Extra LaTeX text to be added after the row. Similar
 #' with `add.to.row` in xtable
 #'
-#' @examples x <- knitr::kable(head(mtcars), "html")
+#' @examples
+#' \dontrun{
+#' x <- knitr::kable(head(mtcars), "html")
 #' row_spec(x, 1:2, bold = TRUE, italic = TRUE)
+#' }
 #'
 #' @export
 row_spec <- function(kable_input, row,
@@ -198,7 +201,9 @@ row_spec_latex <- function(kable_input, row, bold, italic, monospace,
                                      underline, strikeout,
                                      color, background, align, font_size, angle,
                                      hline_after, extra_latex_after)
-    temp_sub <- ifelse(i == 1 & table_info$tabular == "longtable", gsub, sub)
+    temp_sub <- ifelse(i == 1 & (table_info$tabular == "longtable" |
+                                   !is.null(table_info$repeat_header_latex)),
+                       gsub, sub)
     if (length(new_row) == 1) {
       out <- temp_sub(target_row, new_row, out, perl = T)
       table_info$contents[i] <- new_row
@@ -246,10 +251,27 @@ latex_new_row_builder <- function(target_row, table_info,
     })
   }
   if (!is.null(color)) {
+    if (table_info$tabular == "tabu") {
+      warning("Setting full_width = TRUE will turn the table into a tabu ",
+              "environment where colors are not really easily configable ",
+              "with this package. Please consider turn off full_width.")
+    }
     new_row <- lapply(new_row, function(x) {
+      x <- clear_color_latex(x)
       paste0("\\\\textcolor", latex_color(color), "\\{", x, "\\}")
     })
   }
+  if (!is.null(background)) {
+    if (table_info$tabular == "tabu") {
+      warning("Setting full_width = TRUE will turn the table into a tabu ",
+              "environment where colors are not really easily configable ",
+              "with this package. Please consider turn off full_width.")
+    }
+    new_row <- lapply(new_row, function(x) {
+      x <- clear_color_latex(x, background = TRUE)
+      paste0("\\\\cellcolor", latex_color(background), "\\{", x, "\\}")
+    })
+  }
   if (!is.null(font_size)) {
     new_row <- lapply(new_row, function(x) {
       paste0("\\\\begingroup\\\\fontsize\\{", font_size, "\\}\\{",
@@ -282,9 +304,9 @@ latex_new_row_builder <- function(target_row, table_info,
 
   new_row <- paste(unlist(new_row), collapse = " & ")
 
-  if (!is.null(background)) {
-    new_row <- paste0("\\\\rowcolor", latex_color(background), "  ", new_row)
-  }
+  # if (!is.null(background)) {
+  #   new_row <- paste0("\\\\rowcolor", latex_color(background), "  ", new_row)
+  # }
 
   if (!hline_after & is.null(extra_latex_after)) {
     return(new_row)
@@ -301,3 +323,5 @@ latex_new_row_builder <- function(target_row, table_info,
     return(c(new_row, latex_after))
   }
 }
+
+
